package com.yizhi.system.application.service.impl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.yizhi.system.application.domain.Dictionary;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.system.application.mapper.DictionaryMapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.system.application.service.DictionaryService;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author shengchenglong
 * @since 2018-01-04
 */
@Service
@Transactional
public class DictionaryServiceImpl extends ServiceImpl<DictionaryMapper, Dictionary> implements DictionaryService {

    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private DictionaryMapper dictionaryMapper;


    @Override
    public boolean insertDetail(Dictionary dictionary) {
        Dictionary dictExample = new Dictionary();
        dictExample.setCode(dictionary.getCode());
        dictExample.setEnabled(Boolean.TRUE);

        int num = dictionaryMapper.selectCount(new EntityWrapper<Dictionary>(dictExample));
        if (num > 0) {
            throw new RuntimeException("Code has alredy existed! -- code: " + dictionary.getCode());
        }

        // 设置id、设置默认可用
        dictionary.setId(idGenerator.generate());
        dictionary.setEnabled(Boolean.TRUE);

        // 设置layer、rootId、parentId
        if (null == dictionary.getParentId() || dictionary.getParentId() == 0) {
            dictionary.setLayer(0);
            dictionary.setRootId(new Long(0));
            dictionary.setParentId(new Long(0));
        } else {
            Dictionary dictForParent = new Dictionary();
            dictForParent.setId(dictionary.getParentId());
            dictForParent.setEnabled(Boolean.TRUE);
            Dictionary parent = dictionaryMapper.selectOne(dictForParent);

            dictionary.setLayer(parent.getLayer() + 1);
            if (parent.getParentId() == 0) { // 若父对象的parentId为0，则父为root
                dictionary.setRootId(parent.getId());
            } else { // 父对象parentId不为0，则父为非root，父rootId有真实值
                dictionary.setRootId(parent.getRootId());
            }
        }

        // 设置排序
        if (null == dictionary.getSort()) {
            Dictionary layerDict = new Dictionary();
            layerDict.setLayer(dictionary.getLayer());
            layerDict.setRootId(dictionary.getRootId());
            layerDict.setEnabled(Boolean.TRUE);

            int layerCount = dictionaryMapper.selectCount(new EntityWrapper<Dictionary>(layerDict));
            dictionary.setSort(layerCount + 1);
        }

        return dictionaryMapper.insert(dictionary) == 1;
    }

    @Override
    public Dictionary findByCode(String code) {
        Dictionary dictExample = new Dictionary();
        dictExample.setCode(code);
        dictExample.setEnabled(Boolean.TRUE);

        return dictionaryMapper.selectOne(dictExample);
    }

    @Override
    public List<Dictionary> listRootDictionary() {
        Dictionary dictionary = new Dictionary();
        dictionary.setRootId(new Long(0));
        dictionary.setEnabled(Boolean.TRUE);
        return dictionaryMapper.selectList(new EntityWrapper<Dictionary>(dictionary));
    }

    @Override
    public Page<Dictionary> listByPage(Long parentId, String code, String cnName, String enName, int pageNo, int pageSize, String pageSortFiled, String pageSortType) {
        // 组装page，页数、条数、排序字段、排序方式
        Page<Dictionary> page = new Page<Dictionary>(pageNo, pageSize);
        page.setOrderByField(pageSortFiled);
        page.setAsc((null == pageSortType || "desc".equals(pageSortType)) ? false : true);

        Dictionary dictionary = new Dictionary();
        dictionary.setParentId(parentId);
        dictionary.setCode(code);
        dictionary.setEnabled(Boolean.TRUE);
        EntityWrapper<Dictionary> ew = new EntityWrapper<>(dictionary);
        if (StringUtils.isNotEmpty(cnName)) {
            ew.like("cn_name", cnName);
        }
        if (StringUtils.isNotEmpty(enName)) {
            ew.like("en_name", enName);
        }

        return this.selectPage(page, ew);
    }

    @Override
    public List<Dictionary> listChildren(long id, Boolean includeParent, Integer layer) {
        String idsStr = dictionaryMapper.getDictChildList(id, includeParent, layer, true);
        if (StringUtils.isNotEmpty(idsStr)) {
            String[] ids = idsStr.split(",");
            List<Long> idList = new ArrayList<Long>();
            for (String item : ids) {
                idList.add(new Long(item));
            }
            if(includeParent){
                idList.add(id);
            }
            return this.selectBatchIds(idList);
        }
        return new ArrayList<Dictionary>();
    }

    @Override
    public boolean batchDelete(Long[] ids) {
        // 待删除id
        StringBuilder idsToDeleteSb = new StringBuilder();
        for (int i = 0; i < ids.length; i++) {
            idsToDeleteSb.append("," + dictionaryMapper.getDictChildList(ids[i], true, null, true));
            idsToDeleteSb.append("," +ids[i]);
        }
        String[] idsToDeleteStr = idsToDeleteSb.toString().split(",");

        // 去重
        Set<String> idSet = new HashSet<String>();
        for (String id : idsToDeleteStr) {
            if (StringUtils.isNotEmpty(id)) {
                idSet.add(id);
            }
        }

        List<Dictionary> dictToDelete = new ArrayList<>();
        for (String id : idSet) {
            if (StringUtils.isEmpty(id)) {
                continue;
            }
            Dictionary dictionary = new Dictionary();
            dictionary.setId(new Long(id));
            dictionary.setEnabled(Boolean.FALSE);
            dictToDelete.add(dictionary);
        }
        return this.updateBatchById(dictToDelete);
    }

    @Override
    public boolean chedkHasChild(Long id) {
        Dictionary dictExample = new Dictionary();
        dictExample.setParentId(id);
        dictExample.setEnabled(Boolean.TRUE);

        int num = this.selectCount(new EntityWrapper<Dictionary>(dictExample));
        return num > 0;
    }

    @Override
    public boolean checkCode(String code) {
        Dictionary dictExample = new Dictionary();
        dictExample.setCode(code);
        dictExample.setEnabled(Boolean.TRUE);

        int num = this.selectCount(new EntityWrapper<Dictionary>(dictExample));
        return num > 0;
    }


}
